@@ -52,6 +52,8 @@
 #define currentMaxPosition      MinMaxPositionN[1].value
 #define currentMaxTravel        MaxTravelN[0].value
 
+#define POLLMS  50
+
 std::auto_ptr<RoboFocus> roboFocus(0);
 
  void ISInit()
@@ -111,6 +113,7 @@ std::auto_ptr<RoboFocus> roboFocus(0);
 
 RoboFocus::RoboFocus()
 {
+    setFocuserFeatures(true, true, true, false);
 
 }
 
@@ -261,6 +264,7 @@ bool RoboFocus::Connect()
         return false;
     }
 
+    SetTimer(POLLMS);
     IDMessage(getDeviceName(), "Robofocus is online. Getting focus parameters...");
 
     return true;
@@ -383,8 +387,7 @@ int RoboFocus::ReadResponse(char *buf, int nbytes, int timeout)
   int err_code;
 
   if (isDebug())
-      IDLog("##########################################\n")
-;
+      IDLog("##########################################\n");
   while (totalBytesRead < nbytes)
   {
       //IDLog("Nbytes: %d\n", nbytes);
@@ -412,11 +415,12 @@ int RoboFocus::ReadResponse(char *buf, int nbytes, int timeout)
 
       if((  buf[0]== 'I')||  (  buf[0]== 'O'))
       {
+          int i=0;
 
           if (isDebug())
               IDLog("Moving... with buf[0]=(%c)\n", buf[0]) ;
 
-        for(int i=0; i < 9; i++)
+        for(i=0; i < bytesRead; i++)
         {
             if (isDebug())
                 IDLog("buf[%d]=(%c)\n", i, buf[i]);
@@ -433,8 +437,13 @@ int RoboFocus::ReadResponse(char *buf, int nbytes, int timeout)
             }
         }
 
-         usleep(100000) ;
+        //if (buf[i] != 0x46)
+            //return 9;
 
+        if (i == bytesRead)
+            return bytesRead;
+
+        usleep(100000) ;
 
       }
       else
@@ -701,10 +710,10 @@ int RoboFocus::updateRFPositionAbsolute(double *value)
   if ((ret_read_tmp= SendCommand( rf_cmd)) < 0)
     return ret_read_tmp;
 
-  if (sscanf(rf_cmd, "FD0%5f", &temp) < 1)
+  /*if (sscanf(rf_cmd, "FD0%5f", &temp) < 1)
     return -1;
 
-  *value = (double) temp  ;
+  *value = (double) temp  ;*/
 
   return 0;
 }
@@ -736,13 +745,6 @@ int RoboFocus::updateRFPowerSwitches(int s, int  new_sn, int *cur_s1LL, int *cur
     }
 
 
- /*    if( s== ISS_ON) { */
-
-/*       rf_cmd[new_sn + 4]= '2' ; */
-/*     } else { */
-/*       rf_cmd[new_sn + 4]= '1' ; */
-/*     } */
-
     rf_cmd[8]= 0 ;
 
   if ((ret_read_tmp= SendCommand( rf_cmd)) < 0)
@@ -859,28 +861,6 @@ int RoboFocus::updateRFSetPosition(double *value)
   return 0;
 }
 
-bool RoboFocus::ISNewText (const char *dev, const char *name, char *texts[], char *names[], int n)
-{
-    if(strcmp(dev,getDeviceName())==0)
-    {
-        // ===================================
-        // Port Name
-        // ===================================
-        if (!strcmp(name, PortTP.name) )
-        {
-          if (IUUpdateText(&PortTP, texts, names, n) < 0)
-                return false;
-
-          PortTP.s = IPS_OK;
-          IDSetText (&PortTP, NULL);
-          return true;
-        }
-
-    }
-
-     return INDI::Focuser::ISNewText(dev, name, texts, names, n);
-}
-
 bool RoboFocus::ISNewSwitch (const char *dev, const char *name, ISState *states, char *names[], int n)
 {
     if(strcmp(dev,getDeviceName())==0)
@@ -1391,7 +1371,7 @@ void RoboFocus::GetFocusParams ()
 
 }
 
-bool RoboFocus::Move(FocusDirection dir, int speed, int duration)
+int RoboFocus::Move(FocusDirection dir, int speed, int duration)
 {
     INDI_UNUSED(speed);
     double pos=0;
@@ -1414,21 +1394,19 @@ bool RoboFocus::Move(FocusDirection dir, int speed, int duration)
 
         dt = tv_finish.tv_sec - tv_start.tv_sec + (tv_finish.tv_usec - tv_start.tv_usec)/1e6;
 
-
-
         duration -= dt * 1000;
 
       // IDLog("dt is: %g --- duration is: %d -- pos: %g\n", dt, duration, pos);
     }
 
-   return true;
+   return 0;
 }
 
 
 int RoboFocus::MoveAbs(int targetTicks)
 {
     int ret= -1 ;
-    double new_apos = targetTicks;
+    targetPos = targetTicks;
 
     if (targetTicks < FocusAbsPosN[0].min || targetTicks > FocusAbsPosN[0].max)
     {
@@ -1436,9 +1414,9 @@ int RoboFocus::MoveAbs(int targetTicks)
         return -1;
     }
 
-    IDMessage(getDeviceName() , "Focuser is moving to requested position...");
+    //IDMessage(getDeviceName() , "Focuser is moving to requested position %d...", targetTicks);
 
-    if(( ret= updateRFPositionAbsolute(&new_apos)) < 0)
+    if(( ret= updateRFPositionAbsolute(&targetPos)) < 0)
     {
 
         IDMessage(getDeviceName(), "Read out of the absolute movement failed %3d, trying to recover position.", ret);
@@ -1461,8 +1439,7 @@ int RoboFocus::MoveAbs(int targetTicks)
       return -1;
     }
 
-
-    return 0;
+    return 1;
 }
 
 int RoboFocus::MoveRel(FocusDirection dir, unsigned int ticks)
@@ -1526,3 +1503,48 @@ bool RoboFocus::saveConfigItems(FILE *fp)
     IUSaveConfigNumber(fp, &SetBacklashNP);
 }
 
+void RoboFocus::TimerHit()
+{
+    if (FocusAbsPosNP.s == IPS_BUSY)
+    {
+        int nbytes_read=0;
+        float tempPos;
+        char rf_cmd[32] ;
+
+        nbytes_read= ReadResponse(rf_cmd, RF_MAX_CMD, RF_TIMEOUT) ;
+
+        rf_cmd[ nbytes_read - 1] = 0 ;
+
+        if (nbytes_read == 9)
+        {
+
+            if (sscanf(rf_cmd, "FD0%5f", &tempPos) > 0)
+            {
+                currentPosition = tempPos;
+                if (currentPosition == targetPos)
+                {
+                    FocusAbsPosNP.s = IPS_OK;
+                    IDSetNumber(&FocusAbsPosNP, NULL);
+                }
+            }
+        }
+
+    }
+
+    SetTimer(POLLMS);
+
+}
+
+bool RoboFocus::Abort()
+{
+    int nbytes_written;
+    const char *buf = "\r";
+    if (tty_write(PortFD, buf, strlen(buf), &nbytes_written) == TTY_OK)
+        return true;
+    else
+        return false;
+}
+
+
+
+
