@@ -53,8 +53,6 @@ const char *OPTIONS_GROUP  = "Options";			// Options Group
 /* Handy Macros */
 #define currentRA	EquatorialCoordsRN[0].value
 #define currentDEC	EquatorialCoordsRN[1].value
-#define targetRA	EquatorialCoordsWN[0].value
-#define targetDEC	EquatorialCoordsWN[1].value
 
 static void ISPoll(void *);
 static void retry_connection(void *);
@@ -160,7 +158,7 @@ IEQ45Basic::IEQ45Basic()
    lastDEC	  = 0;
    currentSet     = 0;
 
-   IDLog("Initilizing from IEQ45 device...\n");
+   IDLog("Initializing from IEQ45 device...\n");
    IDLog("Driver Version: 0.1 (2011-11-07)\n");
  
    enable_simulation(false);  
@@ -191,7 +189,7 @@ void IEQ45Basic::init_properties()
     IUFillSwitchVector(&OnCoordSetSP, OnCoordSetS, NARRAY(OnCoordSetS), mydev, "ON_COORD_SET", "On Set", BASIC_GROUP, IP_RW, ISR_1OFMANY, 0, IPS_IDLE);
 
     //Track MODE
-    IUFillSwitch(&TrackModeS[0],"SIDERAL", "Sideral", ISS_ON);
+    IUFillSwitch(&TrackModeS[0],"SIDERAL", "Sidereal", ISS_ON);
     IUFillSwitch(&TrackModeS[1],"LUNAR","Lunar", ISS_OFF);
     IUFillSwitch(&TrackModeS[2],"SOLAR", "Solar", ISS_OFF);
     IUFillSwitch(&TrackModeS[3],"ZERO", "Stop", ISS_OFF);
@@ -205,15 +203,10 @@ void IEQ45Basic::init_properties()
     IUFillText(&PortT[0], "PORT", "Port", "/dev/ttyS0");
     IUFillTextVector(&PortTP, PortT, NARRAY(PortT), mydev, "DEVICE_PORT", "Ports", BASIC_GROUP, IP_RW, 0, IPS_IDLE);
 
-    // Equatorial Coords - SET
-    IUFillNumber(&EquatorialCoordsWN[0], "RA", "RA  H:M:S", "%10.6m",  0., 24., 0., 0.);
-    IUFillNumber(&EquatorialCoordsWN[1], "DEC", "Dec D:M:S", "%10.6m", -90., 90., 0., 0.);
-    IUFillNumberVector(&EquatorialCoordsWNP, EquatorialCoordsWN, NARRAY(EquatorialCoordsWN), mydev, "EQUATORIAL_EOD_COORD_REQUEST" , "Equatorial JNow", BASIC_GROUP, IP_RW, 0, IPS_IDLE);
-
-    // Equatorial Coords - READ
+    // Equatorial Coords
     IUFillNumber(&EquatorialCoordsRN[0], "RA", "RA  H:M:S", "%10.6m",  0., 24., 0., 0.);
     IUFillNumber(&EquatorialCoordsRN[1], "DEC", "Dec D:M:S", "%10.6m", -90., 90., 0., 0.);
-    IUFillNumberVector(&EquatorialCoordsRNP, EquatorialCoordsRN, NARRAY(EquatorialCoordsRN), mydev, "EQUATORIAL_EOD_COORD" , "Equatorial JNow", BASIC_GROUP, IP_RO, 0, IPS_IDLE);
+    IUFillNumberVector(&EquatorialCoordsRNP, EquatorialCoordsRN, NARRAY(EquatorialCoordsRN), mydev, "EQUATORIAL_EOD_COORD" , "Equatorial JNow", BASIC_GROUP, IP_RW, 0, IPS_IDLE);
 
 }
 
@@ -229,7 +222,6 @@ void IEQ45Basic::ISGetProperties(const char *dev)
   // Main Control
   IDDefSwitch(&ConnectSP, NULL);
   IDDefText(&PortTP, NULL);
-  IDDefNumber(&EquatorialCoordsWNP, NULL);
   IDDefNumber(&EquatorialCoordsRNP, NULL);
   IDDefSwitch(&OnCoordSetSP, NULL);
   IDDefSwitch(&TrackModeSP, NULL);
@@ -290,19 +282,19 @@ void IEQ45Basic::ISNewNumber (const char *dev, const char *name, double values[]
 	// ===================================
         // Equatorial Coords
 	// ===================================
-	if (!strcmp (name, EquatorialCoordsWNP.name))
+    if (!strcmp (name, EquatorialCoordsRNP.name))
 	{
 	  int i=0, nset=0, error_code=0;
 	  double newRA =0, newDEC =0;
 
 	    for (nset = i = 0; i < n; i++)
 	    {
-		INumber *eqp = IUFindNumber (&EquatorialCoordsWNP, names[i]);
-		if (eqp == &EquatorialCoordsWN[0])
+        INumber *eqp = IUFindNumber (&EquatorialCoordsRNP, names[i]);
+        if (eqp == &EquatorialCoordsRN[0])
 		{
                     newRA = values[i];
 		    nset += newRA >= 0 && newRA <= 24.0;
-		} else if (eqp == &EquatorialCoordsWN[1])
+        } else if (eqp == &EquatorialCoordsRN[1])
 		{
 		    newDEC = values[i];
 		    nset += newDEC >= -90.0 && newDEC <= 90.0;
@@ -323,7 +315,7 @@ void IEQ45Basic::ISNewNumber (const char *dev, const char *name, double values[]
 	   
 	   if (!simulation && ( (error_code = setObjectRA(fd, newRA)) < 0 || ( error_code = setObjectDEC(fd, newDEC)) < 0))
 	   {
-	     handle_error(&EquatorialCoordsWNP, error_code, "Setting RA/DEC");
+         handle_error(&EquatorialCoordsRNP, error_code, "Setting RA/DEC");
 	     return;
 	   } 
 	   
@@ -332,15 +324,15 @@ void IEQ45Basic::ISNewNumber (const char *dev, const char *name, double values[]
 	   
 	   if (process_coords() == false)
 	   {
-	     EquatorialCoordsWNP.s = IPS_ALERT;
-	     IDSetNumber(&EquatorialCoordsWNP, NULL);
+         EquatorialCoordsRNP.s = IPS_ALERT;
+         IDSetNumber(&EquatorialCoordsRNP, NULL);
 	     
 	   }
 	} // end nset
 	else
 	{
-		EquatorialCoordsWNP.s = IPS_ALERT;
-		IDSetNumber(&EquatorialCoordsWNP, "RA or Dec missing or invalid");
+        EquatorialCoordsRNP.s = IPS_ALERT;
+        IDSetNumber(&EquatorialCoordsRNP, "RA or Dec missing or invalid");
 	}
 
 	    return;
@@ -412,15 +404,13 @@ void IEQ45Basic::ISNewSwitch (const char *dev, const char *name, ISState *states
 	  IUResetSwitch(&AbortSlewSP);
 	  abortSlew(fd);
 
-	    if (EquatorialCoordsWNP.s == IPS_BUSY)
+        if (EquatorialCoordsRNP.s == IPS_BUSY)
 	    {
-		AbortSlewSP.s = IPS_OK;
-		EquatorialCoordsWNP.s       = IPS_IDLE;
-                EquatorialCoordsRNP.s       = IPS_IDLE;
-		IDSetSwitch(&AbortSlewSP, "Slew aborted.");
-		IDSetNumber(&EquatorialCoordsWNP, NULL);
-		IDSetNumber(&EquatorialCoordsRNP, NULL);
-            }
+            AbortSlewSP.s = IPS_OK;
+            EquatorialCoordsRNP.s       = IPS_IDLE;
+            IDSetSwitch(&AbortSlewSP, "Slew aborted.");
+            IDSetNumber(&EquatorialCoordsRNP, NULL);
+        }
 
 	    return;
 	}
@@ -472,10 +462,8 @@ void IEQ45Basic::reset_all_properties()
     TrackModeSP.s		= IPS_IDLE;  
     AbortSlewSP.s		= IPS_IDLE;
     PortTP.s			= IPS_IDLE;
-    EquatorialCoordsWNP.s	= IPS_IDLE;
     EquatorialCoordsRNP.s	= IPS_IDLE;
 
-
     IUResetSwitch(&OnCoordSetSP);
     IUResetSwitch(&TrackModeSP);
     IUResetSwitch(&AbortSlewSP);
@@ -490,7 +478,6 @@ void IEQ45Basic::reset_all_properties()
     IDSetSwitch(&TrackModeSP, NULL);
     IDSetSwitch(&AbortSlewSP, NULL);
     IDSetText(&PortTP, NULL);
-    IDSetNumber(&EquatorialCoordsWNP, NULL);
     IDSetNumber(&EquatorialCoordsRNP, NULL);
 }
 
@@ -537,7 +524,7 @@ void IEQ45Basic::ISPoll()
         double dx, dy;
 	int error_code=0;
 
-	switch (EquatorialCoordsWNP.s)
+    switch (EquatorialCoordsRNP.s)
 	{
 		case IPS_IDLE:
 		getIEQ45RA(fd, &currentRA);
@@ -564,9 +551,7 @@ void IEQ45Basic::ISPoll()
 		        lastDEC = currentDEC;
 	       		IUResetSwitch(&OnCoordSetSP);
 	       		OnCoordSetSP.s = IPS_OK;
-	       		EquatorialCoordsWNP.s = IPS_OK;
 			EquatorialCoordsRNP.s = IPS_OK;
-	       		IDSetNumber (&EquatorialCoordsWNP, NULL);
 			IDSetNumber (&EquatorialCoordsRNP, NULL);
 
 			switch (currentSet)
@@ -627,9 +612,9 @@ bool IEQ45Basic::process_coords()
     // Slew
     case IEQ45_SLEW:
           lastSet = IEQ45_SLEW;
-	  if (EquatorialCoordsWNP.s == IPS_BUSY)
+      if (EquatorialCoordsRNP.s == IPS_BUSY)
 	  {
-	     IDLog("Aboring Slew\n");
+	     IDLog("Aborting Slew\n");
 	     abortSlew(fd);
 
 	     // sleep for 100 mseconds
@@ -642,12 +627,10 @@ bool IEQ45Basic::process_coords()
 	    return false;
 	  }
 
-	  EquatorialCoordsWNP.s = IPS_BUSY;
 	  EquatorialCoordsRNP.s = IPS_BUSY;
 	  fs_sexa(RAStr, targetRA, 2, 3600);
 	  fs_sexa(DecStr, targetDEC, 2, 3600);
-	  IDSetNumber(&EquatorialCoordsWNP, "Slewing to JNow RA %s - DEC %s", RAStr, DecStr);
-	  IDSetNumber(&EquatorialCoordsRNP, NULL);
+      IDSetNumber(&EquatorialCoordsRNP, "Slewing to JNow RA %s - DEC %s", RAStr, DecStr);
 	  IDLog("Slewing to JNow RA %s - DEC %s\n", RAStr, DecStr);
 	  break;
 
@@ -655,25 +638,23 @@ bool IEQ45Basic::process_coords()
     // Sync
     case IEQ45_SYNC:
           lastSet = IEQ45_SYNC;
-	  EquatorialCoordsWNP.s = IPS_IDLE;
+      EquatorialCoordsRNP.s = IPS_IDLE;
 	   
 	  if ( !simulation && ( error_code = Sync(fd, syncString) < 0) )
 	  {
-	        IDSetNumber( &EquatorialCoordsWNP , "Synchronization failed.");
+            IDSetNumber( &EquatorialCoordsRNP , "Synchronization failed.");
 		return false;
 	  }
 
 	  if (simulation)
 	  {
-             EquatorialCoordsRN[0].value = EquatorialCoordsWN[0].value;
-             EquatorialCoordsRN[1].value = EquatorialCoordsWN[1].value;
+             EquatorialCoordsRN[0].value = EquatorialCoordsRN[0].value;
+             EquatorialCoordsRN[1].value = EquatorialCoordsRN[1].value;
 	  }
 
-	  EquatorialCoordsWNP.s = IPS_OK;
 	  EquatorialCoordsRNP.s = IPS_OK;
-	  IDSetNumber(&EquatorialCoordsRNP, NULL);
 	  IDLog("Synchronization successful %s\n", syncString);
-	  IDSetNumber(&EquatorialCoordsWNP, "Synchronization successful.");
+      IDSetNumber(&EquatorialCoordsRNP, "Synchronization successful.");
 	  break;
     }
 
@@ -769,7 +750,7 @@ void IEQ45Basic::get_initial_data()
 void IEQ45Basic::slew_error(int slewCode)
 {
     OnCoordSetSP.s = IPS_IDLE;
-    IDLog("Aboring Slew\n");
+    IDLog("Aborting Slew\n");
     abortSlew(fd);
     if (slewCode == 1)
 	IDSetSwitch (&OnCoordSetSP, "Object below horizon.");
