@@ -38,6 +38,7 @@
 #include "lx200gps.h"
 #include "lx200_16.h"
 #include "lx200classic.h"
+#include "lx200ap.h"
 
 // We declare an auto pointer to LX200Generic.
 std::auto_ptr<LX200Generic> telescope(0);
@@ -98,6 +99,12 @@ void ISInit()
   
    if(telescope.get() == 0) telescope.reset(new LX200Autostar());
 
+ }
+ else if (strstr(me, "indi_lx200ap"))
+ {
+    IDLog("initializing from Astrophysics device...\n");
+
+    if(telescope.get() == 0) telescope.reset(new LX200AstroPhysics());
 
  }
  // be nice and give them a generic device
@@ -173,6 +180,11 @@ LX200Generic::~LX200Generic()
     delete(controller);
 }
 
+void LX200Generic::debugTriggered(bool enable)
+{
+   setLX200Debug(enable ? 1 : 0);
+}
+
 const char * LX200Generic::getDefaultName()
 {
     return (char *)"LX200 Generic";
@@ -301,6 +313,8 @@ bool LX200Generic::updateProperties()
         defineSwitch(&FocusModeSP);
 
         getBasicData();
+
+        loadConfig(true);
     }
     else
     {
@@ -729,7 +743,7 @@ bool LX200Generic::updateTime(ln_date * utc, double utc_offset)
 
     JD = ln_get_julian_day(utc);
 
-    DEBUGF(INDI::Logger::DBG_DEBUG, "New JD is %f\n", (float) JD);
+    DEBUGF(INDI::Logger::DBG_DEBUG, "New JD is %f", (float) JD);
 
         // Set Local Time
         if (setLocalTime(PortFD, ltm.hours, ltm.minutes, ltm.seconds) < 0)
@@ -1690,10 +1704,9 @@ bool LX200Generic::canSync()
     return true;
 }
 
-// Don't assume generic can, let children decide
 bool LX200Generic::canPark()
 {
-    return false;
+    return true;
 }
 
 bool LX200Generic::ISSnoopDevice(XMLEle *root)
