@@ -71,12 +71,13 @@ class LX200Generic: public INDI::Telescope, public INDI::GuiderInterface
 
     virtual bool saveConfigItems(FILE *fp);
 
-    bool Goto(double,double);
-    bool Park();
-    bool Sync(double ra, double dec);
+    virtual bool Goto(double,double);
+    virtual bool Park();
+    virtual bool Sync(double ra, double dec);
     virtual bool canSync();
     virtual bool canPark();
 
+    virtual void debugTriggered(bool enable);
 
     virtual void getBasicData();
     void slewError(int slewCode);
@@ -88,9 +89,9 @@ class LX200Generic: public INDI::Telescope, public INDI::GuiderInterface
     static void updateFocusHelper(void *p);
     static void guideTimeoutHelper(void *p);
 
-    void processNSWE(double mag, double angle);
-    void processJoystick(const char * joystick_n, double mag, double angle);
-    void processButton(const char * button_n, ISState state);
+    virtual void processNSWE(double mag, double angle);
+    virtual void processJoystick(const char * joystick_n, double mag, double angle);
+    virtual void processButton(const char * button_n, ISState state);
 
     int    GuideNSTID;
     int    GuideWETID;
