@@ -46,6 +46,8 @@ public:
         double targetRA;
         double targetDEC;
 
+        unsigned int DBG_SCOPE;
+
         bool Parked;
 
         double guiderEWTarget[2];
@@ -54,17 +56,14 @@ public:
         INumber GuideRateN[2];
         INumberVectorProperty GuideRateNP;
 
-        INumberVectorProperty EqPECNV;
-        INumber EqPECN[2];
-
-        ISwitch PECErrNSS[2];
-        ISwitchVectorProperty PECErrNSSP;
-
-        ISwitch PECErrWES[2];
-        ISwitchVectorProperty PECErrWESP;
-
+        INumberVectorProperty EqPENV;
+        INumber EqPEN[2];
 
+        ISwitch PEErrNSS[2];
+        ISwitchVectorProperty PEErrNSSP;
 
+        ISwitch PEErrWES[2];
+        ISwitchVectorProperty PEErrWESP;
 
 };
 
