@@ -28,6 +28,7 @@
 #include <errno.h>
 #include <time.h>
 #include <unistd.h>
+#include <locale.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -40,6 +41,48 @@
 
 #define MAXRBUF 2048
 
+/* output a string expanding special characters into xml/html escape sequences */
+/* N.B. You must free the returned buffer after use! */
+char * escapeXML(const char *s, unsigned int MAX_BUF_SIZE)
+{
+        char *buf = malloc(sizeof(char)*MAX_BUF_SIZE);
+        char *out = buf;
+        unsigned int i=0;
+
+        for (i=0; i <= strlen(s); i++)
+        {
+            switch (s[i])
+            {
+                case '&':
+                    strncpy(out, "&amp;", 5);
+                    out+=5;
+                    break;
+                case '\'':
+                    strncpy(out, "&apos;", 6);
+                    out+=6;
+                    break;
+                case '"':
+                    strncpy(out, "&quot;", 6);
+                    out+=6;
+                    break;
+                case '<':
+                    strncpy(out, "&lt;", 4);
+                    out+=4;
+                    break;
+                case '>':
+                    strncpy(out, "&gt;", 4);
+                    out+=4;
+                    break;
+                default:
+                    strncpy(out++, s+i, 1);
+                    break;
+            }
+
+        }
+
+        return buf;
+}
+
 /* tell Client to delete the property with given name on given device, or
  * entire device if !name
  */
@@ -225,7 +268,7 @@ int IUUpdateNumber(INumberVectorProperty *nvp, double values[], char *names[], i
     if (values[i] < np->min || values[i] > np->max)
     {
        nvp->s = IPS_IDLE;
-       IDSetNumber(nvp, "Error: Invalid range. Valid range is from %g to %g", np->min, np->max);
+       IDSetNumber(nvp, "Error: Invalid range for %s. Valid range is from %g to %g. Requested value is %g", np->name, np->min, np->max, values[i]);
        return -1;
     }
       
@@ -312,28 +355,42 @@ int IUSaveBLOB(IBLOB *bp, int size, int blobsize, char *blob, char *format)
 
 void IUFillSwitch(ISwitch *sp, const char *name, const char * label, ISState s)
 {
-  strncpy(sp->name, name, MAXINDINAME);
-  strncpy(sp->label, label, MAXINDILABEL);
+  char *escapedName = escapeXML(name, MAXINDINAME);
+  char *escapedLabel = escapeXML(label, MAXINDILABEL);
+
+  strncpy(sp->name, escapedName, MAXINDINAME);
+  strncpy(sp->label, escapedLabel, MAXINDILABEL);
   sp->s = s;
   sp->svp = NULL;
   sp->aux = NULL;
+
+  free(escapedName);
+  free(escapedLabel);
 }
 
 void IUFillLight(ILight *lp, const char *name, const char * label, IPState s)
 {
-  strncpy(lp->name, name, MAXINDINAME);
-  strncpy(lp->label, label, MAXINDILABEL);
+  char *escapedName = escapeXML(name, MAXINDINAME);
+  char *escapedLabel = escapeXML(label, MAXINDILABEL);
+
+  strncpy(lp->name, escapedName, MAXINDINAME);
+  strncpy(lp->label, escapedLabel, MAXINDILABEL);
   lp->s = s;
   lp->lvp = NULL;
   lp->aux = NULL;
+
+  free(escapedName);
+  free(escapedLabel);
 }
 
 
 void IUFillNumber(INumber *np, const char *name, const char * label, const char *format, double min, double max, double step, double value)
 {
+  char *escapedName = escapeXML(name, MAXINDINAME);
+  char *escapedLabel = escapeXML(label, MAXINDILABEL);
 
-  strncpy(np->name, name, MAXINDINAME);
-  strncpy(np->label, label, MAXINDILABEL);
+  strncpy(np->name, escapedName, MAXINDINAME);
+  strncpy(np->label, escapedLabel, MAXINDILABEL);
   strncpy(np->format, format, MAXINDIFORMAT);
   
   np->min	= min;
@@ -343,13 +400,18 @@ void IUFillNumber(INumber *np, const char *name, const char * label, const char
   np->nvp	= NULL;
   np->aux0	= NULL;
   np->aux1	= NULL;
+
+  free(escapedName);
+  free(escapedLabel);
 }
 
 void IUFillText(IText *tp, const char *name, const char * label, const char *initialText)
 {
+  char *escapedName = escapeXML(name, MAXINDINAME);
+  char *escapedLabel = escapeXML(label, MAXINDILABEL);
 
-  strncpy(tp->name, name, MAXINDINAME);
-  strncpy(tp->label, label, MAXINDILABEL);
+  strncpy(tp->name, escapedName, MAXINDINAME);
+  strncpy(tp->label, escapedLabel, MAXINDILABEL);
   tp->text = NULL;
   tp->tvp  = NULL;
   tp->aux0 = NULL;
@@ -358,14 +420,20 @@ void IUFillText(IText *tp, const char *name, const char * label, const char *ini
   if (initialText && strlen(initialText) > 0)
     IUSaveText(tp, initialText);
 
+  free(escapedName);
+  free(escapedLabel);
+
 }
 
 void IUFillBLOB(IBLOB *bp, const char *name, const char * label, const char *format)
 {
+    char *escapedName = escapeXML(name, MAXINDINAME);
+    char *escapedLabel = escapeXML(label, MAXINDILABEL);
+
     memset(bp, 0, sizeof(IBLOB));
-    strncpy(bp->name, name, MAXINDINAME);
-    strncpy(bp->label, label, MAXINDILABEL);
-    strncpy(bp->format, label, MAXINDIBLOBFMT);
+    strncpy(bp->name, escapedName, MAXINDINAME);
+    strncpy(bp->label, escapedLabel, MAXINDILABEL);
+    strncpy(bp->format, format, MAXINDIBLOBFMT);
     bp->blob     = 0;
     bp->bloblen  = 0;
     bp->size     = 0;
@@ -373,13 +441,19 @@ void IUFillBLOB(IBLOB *bp, const char *name, const char * label, const char *for
     bp->aux0     = 0;
     bp->aux1     = 0;
     bp->aux2     = 0;
+
+    free(escapedName);
+    free(escapedLabel);
 }
 
 void IUFillSwitchVector(ISwitchVectorProperty *svp, ISwitch *sp, int nsp, const char * dev, const char *name, const char *label, const char *group, IPerm p, ISRule r, double timeout, IPState s)
 {
+  char *escapedName = escapeXML(name, MAXINDINAME);
+  char *escapedLabel = escapeXML(label, MAXINDILABEL);
+
   strncpy(svp->device, dev, MAXINDIDEVICE);
-  strncpy(svp->name, name, MAXINDINAME);
-  strncpy(svp->label, label, MAXINDILABEL);
+  strncpy(svp->name, escapedName, MAXINDINAME);
+  strncpy(svp->label, escapedLabel, MAXINDILABEL);
   strncpy(svp->group, group, MAXINDIGROUP);
   
   svp->p	= p;
@@ -388,27 +462,40 @@ void IUFillSwitchVector(ISwitchVectorProperty *svp, ISwitch *sp, int nsp, const
   svp->s	= s;
   svp->sp	= sp;
   svp->nsp	= nsp;
+
+  free(escapedName);
+  free(escapedLabel);
+
 }
 
 void IUFillLightVector(ILightVectorProperty *lvp, ILight *lp, int nlp, const char * dev, const char *name, const char *label, const char *group, IPState s)
 {
+    char *escapedName = escapeXML(name, MAXINDINAME);
+    char *escapedLabel = escapeXML(label, MAXINDILABEL);
+
     strncpy(lvp->device, dev, MAXINDIDEVICE);
-    strncpy(lvp->name, name, MAXINDINAME);
-    strncpy(lvp->label, label, MAXINDILABEL);
+    strncpy(lvp->name, escapedName, MAXINDINAME);
+    strncpy(lvp->label, escapedLabel, MAXINDILABEL);
     strncpy(lvp->group, group, MAXINDIGROUP);
     strcpy(lvp->timestamp, "");
   
   lvp->s	= s;
   lvp->lp	= lp;
   lvp->nlp	= nlp;
+
+  free(escapedName);
+  free(escapedLabel);
+
 }
  
 void IUFillNumberVector(INumberVectorProperty *nvp, INumber *np, int nnp, const char * dev, const char *name, const char *label, const char* group, IPerm p, double timeout, IPState s)
 {
-  
+ char *escapedName = escapeXML(name, MAXINDINAME);
+ char *escapedLabel = escapeXML(label, MAXINDILABEL);
+
  strncpy(nvp->device, dev, MAXINDIDEVICE);
- strncpy(nvp->name, name, MAXINDINAME);
- strncpy(nvp->label, label, MAXINDILABEL);
+ strncpy(nvp->name, escapedName, MAXINDINAME);
+ strncpy(nvp->label, escapedLabel, MAXINDILABEL);
  strncpy(nvp->group, group, MAXINDIGROUP);
  strcpy(nvp->timestamp, "");
   
@@ -417,31 +504,43 @@ void IUFillNumberVector(INumberVectorProperty *nvp, INumber *np, int nnp, const
   nvp->s	= s;
   nvp->np	= np;
   nvp->nnp	= nnp;
+
+  free(escapedName);
+  free(escapedLabel);
+
   
 }
 
 void IUFillTextVector(ITextVectorProperty *tvp, IText *tp, int ntp, const char * dev, const char *name, const char *label, const char* group, IPerm p, double timeout, IPState s)
 {
+    char *escapedName = escapeXML(name, MAXINDINAME);
+    char *escapedLabel = escapeXML(label, MAXINDILABEL);
+
     strncpy(tvp->device, dev, MAXINDIDEVICE);
-    strncpy(tvp->name, name, MAXINDINAME);
-    strncpy(tvp->label, label, MAXINDILABEL);
+    strncpy(tvp->name, escapedName, MAXINDINAME);
+    strncpy(tvp->label, escapedLabel, MAXINDILABEL);
     strncpy(tvp->group, group, MAXINDIGROUP);
     strcpy(tvp->timestamp, "");
   
-  tvp->p	= p;
-  tvp->timeout	= timeout;
-  tvp->s	= s;
-  tvp->tp	= tp;
-  tvp->ntp	= ntp;
+    tvp->p	= p;
+    tvp->timeout	= timeout;
+    tvp->s	= s;
+    tvp->tp	= tp;
+    tvp->ntp	= ntp;
 
+    free(escapedName);
+    free(escapedLabel);
 }
 
 void IUFillBLOBVector(IBLOBVectorProperty *bvp, IBLOB *bp, int nbp, const char * dev, const char *name, const char *label, const char* group, IPerm p, double timeout, IPState s)
 {
+    char *escapedName = escapeXML(name, MAXINDINAME);
+    char *escapedLabel = escapeXML(label, MAXINDILABEL);
+
     memset(bvp, 0, sizeof(IBLOBVectorProperty));
     strncpy(bvp->device, dev, MAXINDIDEVICE);
-    strncpy(bvp->name, name, MAXINDINAME);
-    strncpy(bvp->label, label, MAXINDILABEL);
+    strncpy(bvp->name, escapedName, MAXINDINAME);
+    strncpy(bvp->label, escapedLabel, MAXINDILABEL);
     strncpy(bvp->group, group, MAXINDIGROUP);
     strcpy(bvp->timestamp, "");
 
@@ -450,6 +549,9 @@ void IUFillBLOBVector(IBLOBVectorProperty *bvp, IBLOB *bp, int nbp, const char *
     bvp->s	= s;
     bvp->bp	= bp;
     bvp->nbp	= nbp;
+
+    free(escapedName);
+    free(escapedLabel);
 }
 
 /*****************************************************************************
@@ -698,6 +800,8 @@ clientMsgCB (int fd, void *arg)
 int
 dispatch (XMLEle *root, char msg[])
 {
+
+
         char *rtag = tagXMLEle(root);
         XMLEle *ep;
         int n,i=0;
@@ -717,6 +821,16 @@ dispatch (XMLEle *root, char msg[])
             if (crackDN (root, &dev, &name, msg) < 0)
                 return (-1);
 
+            /* ensure property is not RO */
+            for (i=0; i < nroCheck; i++)
+            {
+              if (!strcmp(roCheck[i].propName, name))
+              {
+               if (roCheck[i].perm == IP_RO)
+                 return -1;
+              }
+            }
+
             /* seed for reallocs */
             if (!doubles) {
                 doubles = (double *) malloc (1);
@@ -724,6 +838,7 @@ dispatch (XMLEle *root, char msg[])
             }
 
             /* pull out each name/value pair */
+            setlocale(LC_NUMERIC,"C");
             for (n = 0, ep = nextXMLEle(root,1); ep; ep = nextXMLEle(root,0)) {
                 if (strcmp (tagXMLEle(ep), "oneNumber") == 0) {
                     XMLAtt *na = findXMLAtt (ep, "name");
@@ -743,16 +858,7 @@ dispatch (XMLEle *root, char msg[])
                     }
                 }
             }
-
-            /* insure property is not RO */
-            for (i=0; i < nroCheck; i++)
-            {
-              if (!strcmp(roCheck[i].propName, name))
-              {
-               if (roCheck[i].perm == IP_RO)
-                 return -1;
-              }
-            }
+            setlocale(LC_NUMERIC,"");
 
             /* invoke driver if something to do, but not an error if not */
             if (n > 0)
@@ -773,6 +879,16 @@ dispatch (XMLEle *root, char msg[])
             if (crackDN (root, &dev, &name, msg) < 0)
                 return (-1);
 
+            /* ensure property is not RO */
+            for (i=0; i < nroCheck; i++)
+            {
+              if (!strcmp(roCheck[i].propName, name))
+              {
+               if (roCheck[i].perm == IP_RO)
+                 return -1;
+              }
+            }
+
             /* seed for reallocs */
             if (!states) {
                 states = (ISState *) malloc (1);
@@ -805,16 +921,6 @@ dispatch (XMLEle *root, char msg[])
                 }
             }
 
-            /* insure property is not RO */
-            for (i=0; i < nroCheck; i++)
-            {
-              if (!strcmp(roCheck[i].propName, name))
-              {
-               if (roCheck[i].perm == IP_RO)
-                 return -1;
-              }
-            }
-
             /* invoke driver if something to do, but not an error if not */
             if (n > 0)
                 ISNewSwitch (dev, name, states, names, n);
@@ -833,6 +939,16 @@ dispatch (XMLEle *root, char msg[])
             if (crackDN (root, &dev, &name, msg) < 0)
                 return (-1);
 
+            /* ensure property is not RO */
+            for (i=0; i < nroCheck; i++)
+            {
+              if (!strcmp(roCheck[i].propName, name))
+              {
+               if (roCheck[i].perm == IP_RO)
+                 return -1;
+              }
+            }
+
             /* seed for reallocs */
             if (!texts) {
                 texts = (char **) malloc (1);
@@ -856,16 +972,6 @@ dispatch (XMLEle *root, char msg[])
                 }
             }
 
-            /* insure property is not RO */
-            for (i=0; i < nroCheck; i++)
-            {
-              if (!strcmp(roCheck[i].propName, name))
-              {
-               if (roCheck[i].perm == IP_RO)
-                 return -1;
-              }
-            }
-
             /* invoke driver if something to do, but not an error if not */
             if (n > 0)
                 ISNewText (dev, name, texts, names, n);
@@ -1002,7 +1108,7 @@ int IUReadConfig(const char *filename, const char *dev, char errmsg[])
     fp = fopen(configFileName, "r");
     if (fp == NULL)
     {
-         snprintf(errmsg, MAXRBUF, "Unable to read config file. Error loading file %s: %s\n", filename, strerror(errno));
+         snprintf(errmsg, MAXRBUF, "Unable to read config file. Error loading file %s: %s\n", configFileName, strerror(errno));
          return -1;
     }
 
@@ -1011,6 +1117,7 @@ int IUReadConfig(const char *filename, const char *dev, char errmsg[])
     if (fproot == NULL)
     {
         snprintf(errmsg, MAXRBUF, "Unable to parse config XML: %s", errmsg);
+        fclose(fp);
         return -1;
     }
 
@@ -1019,7 +1126,10 @@ int IUReadConfig(const char *filename, const char *dev, char errmsg[])
 
         /* pull out device and name */
         if (crackDN (root, &rdev, &rname, errmsg) < 0)
+        {
+            fclose(fp);
             return -1;
+        }
 
         // It doesn't belong to our device??
         if (strcmp(dev, rdev))
@@ -1075,9 +1185,9 @@ void IUSaveDefaultConfig(const char *source_config, const char *dest_config, con
           while((ch = getc(fpin)) != EOF)
             putc(ch, fpout);
 
-          fclose(fpin);
-          fclose(fpout);
+          fclose(fpin);          
        }
+        fclose(fpout);
     }
   }
 
@@ -1164,7 +1274,8 @@ void IUSaveConfigNumber (FILE *fp, const INumberVectorProperty *nvp)
 {
     int i;
 
-    fprintf (fp, "<newNumberVector device='%s' name='%s'>\n", nvp->device, nvp->name);
+    setlocale(LC_NUMERIC,"C");
+   fprintf (fp, "<newNumberVector device='%s' name='%s'>\n", nvp->device, nvp->name);
 
     for (i = 0; i < nvp->nnp; i++)
     {
@@ -1175,6 +1286,7 @@ void IUSaveConfigNumber (FILE *fp, const INumberVectorProperty *nvp)
     }
 
     fprintf (fp, "</newNumberVector>\n");
+    setlocale(LC_NUMERIC,"");
 }
 
 void IUSaveConfigText (FILE *fp, const ITextVectorProperty *tvp)
@@ -1251,6 +1363,7 @@ IDDefText (const ITextVectorProperty *tvp, const char *fmt, ...)
         ROSC *SC;
 
         xmlv1();
+        setlocale(LC_NUMERIC,"C");
         printf ("<defTextVector\n");
         printf ("  device='%s'\n", tvp->device);
         printf ("  name='%s'\n", tvp->name);
@@ -1292,6 +1405,7 @@ IDDefText (const ITextVectorProperty *tvp, const char *fmt, ...)
                 SC->perm = tvp->p;
         }
 
+        setlocale(LC_NUMERIC,"");
         fflush (stdout);
 }
 
@@ -1303,6 +1417,7 @@ IDDefNumber (const INumberVectorProperty *n, const char *fmt, ...)
         ROSC *SC;
 
         xmlv1();
+        setlocale(LC_NUMERIC,"C");
         printf ("<defNumberVector\n");
         printf ("  device='%s'\n", n->device);
         printf ("  name='%s'\n", n->name);
@@ -1355,6 +1470,7 @@ IDDefNumber (const INumberVectorProperty *n, const char *fmt, ...)
 
         }
 
+        setlocale(LC_NUMERIC,"");
         fflush (stdout);
 }
 
@@ -1367,6 +1483,7 @@ IDDefSwitch (const ISwitchVectorProperty *s, const char *fmt, ...)
         ROSC *SC;
 
         xmlv1();
+        setlocale(LC_NUMERIC,"C");
         printf ("<defSwitchVector\n");
         printf ("  device='%s'\n", s->device);
         printf ("  name='%s'\n", s->name);
@@ -1409,6 +1526,7 @@ IDDefSwitch (const ISwitchVectorProperty *s, const char *fmt, ...)
                 SC->perm = s->p;
         }
 
+        setlocale(LC_NUMERIC,"");
         fflush (stdout);
 }
 
@@ -1457,6 +1575,7 @@ IDDefBLOB (const IBLOBVectorProperty *b, const char *fmt, ...)
   ROSC *SC;
 
         xmlv1();
+        setlocale(LC_NUMERIC,"C");
         printf ("<defBLOBVector\n");
         printf ("  device='%s'\n", b->device);
         printf ("  name='%s'\n", b->name);
@@ -1497,6 +1616,7 @@ IDDefBLOB (const IBLOBVectorProperty *b, const char *fmt, ...)
                 SC->perm = b->p;
         }
 
+        setlocale(LC_NUMERIC,"");
         fflush (stdout);
 }
 
@@ -1507,6 +1627,7 @@ IDSetText (const ITextVectorProperty *tvp, const char *fmt, ...)
         int i;
 
         xmlv1();
+        setlocale(LC_NUMERIC,"C");
         printf ("<setTextVector\n");
         printf ("  device='%s'\n", tvp->device);
         printf ("  name='%s'\n", tvp->name);
@@ -1531,6 +1652,7 @@ IDSetText (const ITextVectorProperty *tvp, const char *fmt, ...)
         }
 
         printf ("</setTextVector>\n");
+        setlocale(LC_NUMERIC,"");
         fflush (stdout);
 }
 
@@ -1541,6 +1663,7 @@ IDSetNumber (const INumberVectorProperty *nvp, const char *fmt, ...)
         int i;
 
         xmlv1();
+        setlocale(LC_NUMERIC,"C");
         printf ("<setNumberVector\n");
         printf ("  device='%s'\n", nvp->device);
         printf ("  name='%s'\n", nvp->name);
@@ -1565,6 +1688,7 @@ IDSetNumber (const INumberVectorProperty *nvp, const char *fmt, ...)
         }
 
         printf ("</setNumberVector>\n");
+        setlocale(LC_NUMERIC,"");
         fflush (stdout);
 }
 
@@ -1575,6 +1699,7 @@ IDSetSwitch (const ISwitchVectorProperty *svp, const char *fmt, ...)
         int i;
 
         xmlv1();
+        setlocale(LC_NUMERIC,"C");
         printf ("<setSwitchVector\n");
         printf ("  device='%s'\n", svp->device);
         printf ("  name='%s'\n", svp->name);
@@ -1599,7 +1724,8 @@ IDSetSwitch (const ISwitchVectorProperty *svp, const char *fmt, ...)
         }
 
         printf ("</setSwitchVector>\n");
-        fflush (stdout);
+        setlocale(LC_NUMERIC,"");
+       fflush (stdout);
 }
 
 /* tell client to update an existing lights vector property */
@@ -1642,6 +1768,7 @@ IDSetBLOB (const IBLOBVectorProperty *bvp, const char *fmt, ...)
         int i;
 
         xmlv1();
+        setlocale(LC_NUMERIC,"C");
         printf ("<setBLOBVector\n");
         printf ("  device='%s'\n", bvp->device);
         printf ("  name='%s'\n", bvp->name);
@@ -1678,6 +1805,7 @@ IDSetBLOB (const IBLOBVectorProperty *bvp, const char *fmt, ...)
         }
 
   printf ("</setBLOBVector>\n");
+  setlocale(LC_NUMERIC,"");
   fflush (stdout);
 }
 
@@ -1687,6 +1815,7 @@ void IUUpdateMinMax(const INumberVectorProperty *nvp)
   int i;
 
   xmlv1();
+  setlocale(LC_NUMERIC,"C");
   printf ("<setNumberVector\n");
   printf ("  device='%s'\n", nvp->device);
   printf ("  name='%s'\n", nvp->name);
@@ -1707,6 +1836,7 @@ void IUUpdateMinMax(const INumberVectorProperty *nvp)
   }
 
   printf ("</setNumberVector>\n");
+  setlocale(LC_NUMERIC,"");
   fflush (stdout);
 }
 
