@@ -41,7 +41,7 @@
  * Clients that get more than maxqsiz bytes behind are shut down.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -69,12 +69,18 @@
 #define MAXSBUF     512
 #define	MAXRBUF		4096		/* max read buffering here */
 #define	MAXWSIZ		4096		/* max bytes/write */
-#define	DEFMAXQSIZ	10		/* default max q behind, MB */
+#define	DEFMAXQSIZ	64		/* default max q behind, MB */
+
+#ifdef OSX_HELPER_MODE
+#define LOGNAME "/Users/%s/Library/Logs/indiserver.log"
+#define FIFONAME "/tmp/indiserverFIFO"
+#endif
+
 
 /* associate a usage count with queuded client or device message */
 typedef struct {
     int count;				/* number of consumers left */
-    unsigned int cl;			/* content length */
+    unsigned long cl;			/* content length */
     char *cp;				/* content: buf or malloced */
     char buf[MAXWSIZ];		/* local buf for most messages */
 } Msg;
@@ -122,6 +128,10 @@ static int nclinfo;			/* n total (not active) */
 /* info for each connected driver */
 typedef struct {
     char name[MAXINDINAME];		/* persistent name */
+    char envDev[MAXSBUF];
+    char envConfig[MAXSBUF];
+    char envSkel[MAXSBUF];
+    char envPrefix[MAXSBUF];
     //char dev[MAXINDIDEVICE];		/* device served by this driver */
     char **dev;             /* device served by this driver */
     int ndev;               /* number of devices served by this driver */
@@ -193,12 +203,27 @@ static void Bye(void);
 int
 main (int ac, char *av[])
 {
+
+
 	/* log startup */
 	logStartup(ac, av);
 
 	/* save our name */
 	me = av[0];
 
+#ifdef OSX_HELPER_MODE
+
+  char logname[128];
+  snprintf(logname, 128, LOGNAME, getlogin());
+  fprintf(stderr, "switching stderr to %s", logname);
+  freopen(logname, "w", stderr);
+
+  fifo.name = FIFONAME;
+  verbose = 1;
+  ac = 0;
+
+#else
+
 	/* crack args */
 	while ((--ac > 0) && ((*++av)[0] == '-')) {
 	    char *s;
@@ -228,14 +253,14 @@ main (int ac, char *av[])
 		    port = atoi(*++av);
 		    ac--;
 		    break;
-                case 'f':
-                    if (ac < 2) {
-                        fprintf (stderr, "-f requires fifo node\n");
-                        usage();
-                    }
-                    fifo.name = *++av;
-                    ac--;
-                    break;
+    case 'f':
+        if (ac < 2) {
+            fprintf (stderr, "-f requires fifo node\n");
+            usage();
+        }
+        fifo.name = *++av;
+        ac--;
+        break;
 		case 'v':
 		    verbose++;
 		    break;
@@ -243,6 +268,8 @@ main (int ac, char *av[])
 		    usage();
 		}
 	}
+#endif
+
 
 	/* at this point there are ac args in av[] to name our drivers */
         if (ac == 0 && !fifo.name)
@@ -390,6 +417,10 @@ startLocalDvr (DvrInfo *dp)
 	int rp[2], wp[2], ep[2];
 	int pid;
 
+#ifdef OSX_HELPER_MODE
+  fprintf(stderr, "STARTING \"%s\"\n", dp->name); fflush(stderr);
+#endif
+
 	/* build three pipes: r, w and error*/
 	if (pipe (rp) < 0) {
 	    fprintf (stderr, "%s: read pipe: %s\n", indi_tstamp(NULL),
@@ -424,8 +455,31 @@ startLocalDvr (DvrInfo *dp)
 	    for (fd = 3; fd < 100; fd++)
 		(void) close (fd);
 
-	    /* go -- should never return */
-	    execlp (dp->name, dp->name, NULL);
+	    if (*dp->envDev)
+	      setenv("INDIDEV", dp->envDev, 1);
+	    else
+	      unsetenv("INDIDEV");
+	    if (*dp->envConfig)
+	      setenv("INDICONFIG", dp->envConfig, 1);
+	    else
+	      unsetenv("INDICONFIG");
+	    if (*dp->envSkel)
+	      setenv("INDISKEL", dp->envSkel, 1);
+	    else
+	      unsetenv("INDISKEL");
+	    char executable[MAXSBUF];
+	    if (*dp->envPrefix) {
+	      setenv("INDIPREFIX", dp->envPrefix, 1);
+	      snprintf(executable, MAXSBUF, "%s/bin/%s", dp->envPrefix, dp->name);
+		    execlp (executable, dp->name, NULL);
+      } else {
+	      unsetenv("INDIPREFIX");
+  	    execlp (dp->name, dp->name, NULL);
+	    }
+
+#ifdef OSX_HELPER_MODE
+  fprintf(stderr, "FAILED \"%s\"\n", dp->name); fflush(stderr);
+#endif
 	    fprintf (stderr, "%s: Driver %s: execlp: %s\n", indi_tstamp(NULL),
 						dp->name, strerror(errno));
 	    _exit (1);	/* parent will notice EOF shortly */
@@ -789,16 +843,17 @@ static void newFIFO(void)
      if (verbose)
             fprintf(stderr, "FIFO: %s\n", line);
 
-     char cmd[MAXSBUF], arg[3][1], var[3][MAXSBUF], tDriver[MAXSBUF], tName[MAXSBUF], envDev[MAXSBUF], envConfig[MAXSBUF], envSkel[MAXSBUF];
+     char cmd[MAXSBUF], arg[4][1], var[4][MAXSBUF], tDriver[MAXSBUF], tName[MAXSBUF], envDev[MAXSBUF], envConfig[MAXSBUF], envSkel[MAXSBUF], envPrefix[MAXSBUF];
 
      memset(&tDriver[0], 0, sizeof(MAXSBUF));
      memset(&tName[0], 0, sizeof(MAXSBUF));
      memset(&envDev[0], 0, sizeof(MAXSBUF));
      memset(&envConfig[0], 0, sizeof(MAXSBUF));
      memset(&envSkel[0], 0, sizeof(MAXSBUF));
+     memset(&envPrefix[0], 0, sizeof(MAXSBUF));
 
-     int n = sscanf(line, "%s %s -%1c \"%512[^\"]\" -%1c \"%512[^\"]\" -%1c \"%512[^\"]\"", cmd, tDriver, arg[0], var[0], arg[1], var[1]
-                    , arg[2], var[2]);
+     int n = sscanf(line, "%s %s -%1c \"%512[^\"]\" -%1c \"%512[^\"]\" -%1c \"%512[^\"]\" -%1c \"%512[^\"]\"", cmd, tDriver, arg[0], var[0], arg[1], var[1]
+                    , arg[2], var[2], arg[3], var[3]);
 
 
      int n_args = (n - 2)/2;
@@ -813,33 +868,34 @@ static void newFIFO(void)
          {
              strncpy(tName, var[j], MAXSBUF-1);
              tName[MAXSBUF-1] = '\0';
-             snprintf(envDev, MAXSBUF, "INDIDEV=%s", tName);
+
              if (verbose)
                 fprintf(stderr, "With name: %s\n", envDev);
-
-             putenv(envDev);
          }
          else if (arg[j][0] == 'c')
          {
-             snprintf(envConfig, MAXSBUF, "INDICONFIG=%s", var[j]);
+             strncpy(envConfig, var[j], MAXSBUF-1);
+             envConfig[MAXSBUF-1] = '\0';
 
              if (verbose)
               fprintf(stderr, "With config: %s\n", envConfig);
-
-             putenv(envConfig);
-
          }
          else if (arg[j][0] == 's')
          {
-             snprintf(envSkel, MAXSBUF, "INDISKEL=%s", var[j]);
+             strncpy(envSkel, var[j], MAXSBUF-1);
+             envSkel[MAXSBUF-1] = '\0';
 
              if (verbose)
               fprintf(stderr, "With skeketon: %s\n", envSkel);
-
-             putenv(envSkel);
-
          }
+         else if (arg[j][0] == 'p')
+         {
+             strncpy(envPrefix, var[j], MAXSBUF-1);
+             envPrefix[MAXSBUF-1] = '\0';
 
+             if (verbose)
+              fprintf(stderr, "With prefix: %s\n", envPrefix);
+         }
      }
 
      if (!strcmp(cmd, "start"))
@@ -854,6 +910,10 @@ static void newFIFO(void)
           dp = allocDvr();
           strncpy(dp->name, tDriver, MAXINDIDEVICE);
           //strncpy(dp->dev, tName, MAXINDIDEVICE);
+          strncpy(dp->envDev, tName, MAXSBUF);
+          strncpy(dp->envConfig, envConfig, MAXSBUF);
+          strncpy(dp->envSkel, envSkel, MAXSBUF);
+          strncpy(dp->envPrefix, envPrefix, MAXSBUF);
           startDvr (dp);
     }
     else
@@ -872,8 +932,6 @@ static void newFIFO(void)
            if (verbose)
                fprintf(stderr, "FIFO: Shutting down driver: %s\n", tDriver);
 
-            shutdownDvr(dp, 0);
-
             for (i=0; i < dp->ndev; i++)
             {
                 /* Inform clients that this driver is dead */
@@ -891,6 +949,7 @@ static void newFIFO(void)
               delXMLEle (root);
             }
 
+            shutdownDvr(dp, 0);
          break;
          }
      }
@@ -941,6 +1000,13 @@ newClient()
 			    indi_tstamp(NULL), cp->s, inet_ntoa(addr.sin_addr),
 							ntohs(addr.sin_port));
 	}
+#ifdef OSX_HELPER_MODE
+  int active = 0;
+  for (int i = 0; i < nclinfo; i++)
+    if (clinfo[i].active)
+      active++;
+  fprintf(stderr, "CLIENTS %d\n", active); fflush(stderr);
+#endif
 }
 
 /* read more from the given client, send to each appropriate driver when see
@@ -952,7 +1018,7 @@ readFromClient (ClInfo *cp)
 {
 	char buf[MAXRBUF];
 	int shutany = 0;
-	int i, nr;
+	ssize_t i, nr;
 
 	/* read client */
 	nr = read (cp->s, buf, sizeof(buf));
@@ -1026,7 +1092,7 @@ readFromClient (ClInfo *cp)
 		fprintf (stderr, "%s: Client %d: XML error: %s\n", ts,
 								cp->s, err);
 		fprintf (stderr, "%s: Client %d: XML read: %.*s\n", ts,
-							    cp->s, nr, buf);
+							    cp->s, (int)nr, buf);
 		shutdownClient (cp);
 		return (-1);
 	    }
@@ -1044,7 +1110,7 @@ readFromDriver (DvrInfo *dp)
 {
 	char buf[MAXRBUF];
 	int shutany = 0;
-	int i, nr;
+	ssize_t i, nr;
 
 	/* read driver */
 	nr = read (dp->rfd, buf, sizeof(buf));
@@ -1110,6 +1176,11 @@ readFromDriver (DvrInfo *dp)
             strncpy (dp->dev[dp->ndev], dev, MAXINDIDEVICE-1);
             dp->dev[dp->ndev][MAXINDIDEVICE-1] = '\0';
 
+#ifdef OSX_HELPER_MODE
+            if (!dp->ndev)
+              fprintf(stderr, "STARTED \"%s\"\n", dp->name); fflush(stderr);
+#endif
+
             dp->ndev++;
         }
 
@@ -1139,7 +1210,7 @@ readFromDriver (DvrInfo *dp)
 		fprintf (stderr, "%s: Driver %s: XML error: %s\n", ts,
 								dp->name, err);
 		fprintf (stderr, "%s: Driver %s: XML read: %.*s\n", ts,
-							    dp->name, nr, buf);
+							    dp->name, (int)nr, buf);
                 shutdownDvr (dp, 1);
 		return (-1);
 	    }
@@ -1156,7 +1227,7 @@ stderrFromDriver (DvrInfo *dp)
 {
 	static char exbuf[MAXRBUF];
 	static int nexbuf;
-	int i, nr;
+	ssize_t i, nr;
 
 	/* read more */
 	nr = read (dp->efd, exbuf+nexbuf, sizeof(exbuf)-nexbuf);
@@ -1176,7 +1247,7 @@ stderrFromDriver (DvrInfo *dp)
 	for (i = 0; i < nexbuf; i++) {
 	    if (exbuf[i] == '\n') {
 		fprintf (stderr, "%s: Driver %s: %.*s\n", indi_tstamp(NULL),
-							    dp->name, i, exbuf);
+							    dp->name, (int)i, exbuf);
 		i++;				  /* count including nl */
 		nexbuf -= i;			  /* remove from nexbuf */
 		memmove (exbuf, exbuf+i, nexbuf); /* slide remaining to front */
@@ -1213,6 +1284,13 @@ shutdownClient (ClInfo *cp)
 	if (verbose > 0)
 	    fprintf (stderr, "%s: Client %d: shut down complete - bye!\n",
 							indi_tstamp(NULL), cp->s);
+#ifdef OSX_HELPER_MODE
+  int active = 0;
+  for (int i = 0; i < nclinfo; i++)
+    if (clinfo[i].active)
+      active++;
+  fprintf(stderr, "CLIENTS %d\n", active); fflush(stderr);
+#endif
 }
 
 /* close down the given driver and restart */
@@ -1234,6 +1312,10 @@ shutdownDvr (DvrInfo *dp, int restart)
 	    close (dp->efd);
 	}
 
+#ifdef OSX_HELPER_MODE
+  fprintf(stderr, "STOPPED \"%s\"\n", dp->name); fflush(stderr);
+#endif
+
 	/* free memory */
 	free (dp->sprops);
     free(dp->dev);
@@ -1513,7 +1595,7 @@ freeMsg (Msg *mp)
 static int
 sendClientMsg (ClInfo *cp)
 {
-	int nsend, nw;
+	ssize_t nsend, nw;
 	Msg *mp;
 
 	/* get current message */
@@ -1541,7 +1623,7 @@ sendClientMsg (ClInfo *cp)
 	if (verbose > 2) {
 	    fprintf(stderr, "%s: Client %d: sending msg copy %d nq %d:\n%.*s\n",
 				indi_tstamp(NULL), cp->s, mp->count, nFQ(cp->msgq),
-				nw, &mp->cp[cp->nsent]);
+				(int)nw, &mp->cp[cp->nsent]);
 	} else if (verbose > 1) {
 	    fprintf(stderr, "%s: Client %d: sending %.50s\n", indi_tstamp(NULL),
 						    cp->s, &mp->cp[cp->nsent]);
@@ -1570,7 +1652,7 @@ sendClientMsg (ClInfo *cp)
 static int
 sendDriverMsg (DvrInfo *dp)
 {
-	int nsend, nw;
+	ssize_t nsend, nw;
 	Msg *mp;
 
 	/* get current message */
@@ -1598,7 +1680,7 @@ sendDriverMsg (DvrInfo *dp)
 	if (verbose > 2) {
 	    fprintf(stderr, "%s: Driver %s: sending msg copy %d nq %d:\n%.*s\n",
 			    indi_tstamp(NULL), dp->name, mp->count, nFQ(dp->msgq),
-			    nw, &mp->cp[dp->nsent]);
+			    (int)nw, &mp->cp[dp->nsent]);
 	} else if (verbose > 1) {
 	    fprintf(stderr, "%s: Driver %s: sending %.50s\n", indi_tstamp(NULL),
 						dp->name, &mp->cp[dp->nsent]);
@@ -1642,7 +1724,7 @@ static void
 addClDevice (ClInfo *cp, const char *dev, const char *name, int isblob)
 {
 	Property *pp;
-	char *ip;
+	//char *ip;
         int i=0;
 
         if (isblob)
@@ -1730,7 +1812,7 @@ static void crackBLOBHandling(const char *dev, const char *name, const char *ena
     for (i = 0; i < cp->nprops; i++)
     {
         Property *pp = &cp->props[i];
-        if (!name[0])           
+        if (!name[0])
             crackBLOB(enableBLOB, &pp->blob);
         else if (!strcmp (pp->dev, dev) && (!strcmp(pp->name, name)))
         {
