@@ -21,7 +21,11 @@
 #include <errno.h>
 #include <zlib.h>
 #include <locale.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 
+#include "config.h"
 #include "basedevice.h"
 #include "baseclient.h"
 #include "indicom.h"
@@ -109,7 +113,7 @@ void * INDI::BaseDevice::getRawProperty(const char *name, INDI_TYPE type)
     void *pPtr;
     bool pRegistered = false;
 
-    typename std::vector<INDI::Property *>::iterator orderi = pAll.begin();
+    std::vector<INDI::Property *>::iterator orderi = pAll.begin();
 
     INumberVectorProperty *nvp;
     ITextVectorProperty *tvp;
@@ -328,18 +332,45 @@ int INDI::BaseDevice::removeProperty(const char *name, char *errmsg)
     return INDI_PROPERTY_INVALID;
 }
 
-void INDI::BaseDevice::buildSkeleton(const char *filename)
+bool INDI::BaseDevice::buildSkeleton(const char *filename)
 {
     char errmsg[MAXRBUF];
     FILE *fp = NULL;
     XMLEle *root = NULL, *fproot = NULL;
 
-    fp = fopen(filename, "r");
+    char pathname[MAXRBUF];
+    struct stat st;
+    const char *indiskel = getenv("INDISKEL");
+    if (indiskel) {
+      strncpy(pathname, indiskel, MAXRBUF-1);
+      pathname[MAXRBUF-1] = 0;
+      IDLog("Using INDISKEL %s\n", pathname);
+    } else {
+      if (stat(filename, &st) == 0) {
+        strncpy(pathname, filename, MAXRBUF-1);
+        pathname[MAXRBUF-1] = 0;
+        IDLog("Using %s\n", pathname);
+      } else {
+        const char *slash = strrchr(filename, '/');
+        if (slash)
+          filename = slash + 1;
+        const char *indiprefix = getenv("INDIPREFIX");
+        if (indiprefix) {
+          snprintf(pathname, MAXRBUF-1, "%s/share/indi/%s", indiprefix, filename);
+        } else {
+          snprintf(pathname, MAXRBUF-1, "%s/%s", DATA_INSTALL_DIR, filename);
+        }
+        pathname[MAXRBUF-1] = 0;
+        IDLog("Using prefix %s\n", pathname);
+      }
+    }
+    
+    fp = fopen(pathname, "r");
 
     if (fp == NULL)
     {
-        IDLog("Unable to build skeleton. Error loading file %s: %s\n", filename, strerror(errno));
-        return;
+        IDLog("Unable to build skeleton. Error loading file %s: %s\n", pathname, strerror(errno));
+        return false;
     }
 
     fproot = readXMLFile(fp, lp, errmsg);
@@ -347,7 +378,7 @@ void INDI::BaseDevice::buildSkeleton(const char *filename)
     if (fproot == NULL)
     {
         IDLog("Unable to parse skeleton XML: %s", errmsg);
-        return;
+        return false;
     }
 
     //prXMLEle(stderr, fproot, 0);
@@ -355,6 +386,7 @@ void INDI::BaseDevice::buildSkeleton(const char *filename)
     for (root = nextXMLEle (fproot, 1); root != NULL; root = nextXMLEle (fproot, 0))
             buildProp(root, errmsg);
 
+    return true;
     /**************************************************************************/
 }
 
@@ -393,9 +425,10 @@ int INDI::BaseDevice::buildProp(XMLEle *root, char *errmsg)
         return -1;
     }
 
-
     if (!strcmp (rtag, "defNumberVector"))
     {
+        setlocale(LC_NUMERIC,"C");
+        
         INDI::Property *indiProp = new INDI::Property();
         INumberVectorProperty *nvp = new INumberVectorProperty;
 
@@ -471,6 +504,8 @@ int INDI::BaseDevice::buildProp(XMLEle *root, char *errmsg)
     }
     else
         IDLog("%s: newNumberVector with no valid members\n",rname);
+
+    setlocale(LC_NUMERIC,"");
   }
   else if (!strcmp (rtag, "defSwitchVector"))
   {
@@ -736,7 +771,7 @@ bool INDI::BaseDevice::isConnected()
     if (!sp)
         return false;
 
-    if (sp->s == ISS_ON)
+    if (sp->s == ISS_ON && svp->s == IPS_OK)
         return true;
     else
         return false;
@@ -780,14 +815,16 @@ int INDI::BaseDevice::setValue (XMLEle *root, char * errmsg)
         stateSet = true;
     }
 
-    setlocale(LC_NUMERIC,"C");
-
     /* allow changing the timeout */
     ap = findXMLAtt (root, "timeout");
     if (ap)
     {
+        setlocale(LC_NUMERIC,"C");
+        
         timeout = atof(valuXMLAtt(ap));
         timeoutSet = true;
+
+        setlocale(LC_NUMERIC,"");
     }
 
     checkMessage (root);
@@ -807,6 +844,8 @@ int INDI::BaseDevice::setValue (XMLEle *root, char * errmsg)
         if (timeoutSet)
             nvp->timeout = timeout;
 
+        setlocale(LC_NUMERIC,"C");
+        
        for (ep = nextXMLEle (root, 1); ep != NULL; ep = nextXMLEle (root, 0))
         {
            INumber *np =  IUFindNumber(nvp, findXMLAttValu(ep, "name"));
@@ -957,7 +996,11 @@ int INDI::BaseDevice::setBLOB(IBLOBVectorProperty *bvp, XMLEle * root, char * er
 
                 /* Blob size = 0 when only state changes */
                 if (blobEL->size == 0)
+                {
+                    if (mediator)
+                       mediator->newBLOB(blobEL);
                     continue;
+                }
 
                  blobEL->blob = (unsigned char *) realloc (blobEL->blob, 3*pcdatalenXMLEle(ep)/4);
 
