@@ -73,6 +73,11 @@ extern const char *FOCUS_TAB;
  */
 extern const char *GUIDE_TAB;
 
+/**
+ * @brief ALIGNMENT_TAB Where all the properties for guiding are located.
+ */
+extern const char *ALIGNMENT_TAB;
+
 /**
  * \class INDI::DefaultDevice
    \brief Class to provide extended functionality for devices in addition
@@ -229,9 +234,10 @@ protected:
     // Configuration
 
     /** \brief Load the last saved configuration file
+     *  \param silent if true, don't report any error or messages.
         \return True if successful, false otherwise.
     */
-    virtual bool loadConfig();
+    virtual bool loadConfig(bool silent=false);
 
     /** \brief Save the current properties in a configuration file
         \return True if successful, false otherwise.
